==> daemonshogi-0.1.3.tar.gz: daemonshogi-0.1.3/src/si/comaccess.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * comaccess.c
 * $Source: /cvs/si/comaccess.c,v $
 * $Id: comaccess.c,v 1.9 2002/09/17 20:01:20 mik Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include "comaccess.h"
#include "si.h"
#include "ui.h"

#ifdef USE_SERIAL

#ifndef VSWTC
#define VSWTC 7
#endif /* VSWTC */

ssize_t wait_read(int fd, void *buf, size_t size);

/**
 * ݤCOMACCESSΥݥ󥿤֤
 * @return COMACCESSEΥݥ
 */
COMACCESS *newCOMACCESS(void) {
  COMACCESS *ca;

  ca = (COMACCESS *) malloc(sizeof(COMACCESS));
  if (ca == NULL) {
    printf("No enough memory. In tree.c/newTREE()");
    abort();
  }

  initCOMACCESS(ca);

  return ca;
}

/**
 * ca Υ롣
 * @param bo оݤCOMACCESS
 */
void freeCOMACCESS(COMACCESS *ca) {
#ifdef DEBUG
  assert( ca != NULL );
#endif /* DEBUG */
   
  free(ca); 
}

/**
 * ca 롣
 * @param ca оݤCOMACCESS
 */
void initCOMACCESS(COMACCESS *ca) {
   static const char s[] = "/dev/cua0";
   
   strncpy(ca->DEVICENAME, s, strlen(s));
   ca->DEVICENAME[strlen(s)] = '\0';
}

/**
 * ǥХ̾򥻥åȤ롣
 * @param ca оݤCOMACCESS
 * @param str ǥХ̾
 */
void caSetDEVICENAME(COMACCESS *ca, char *str) {
   strncpy(ca->DEVICENAME, str, strlen(str));
   ca->DEVICENAME[strlen(str)] = '\0';
}

/**
 * ̿жɤϤ褦ꤹ롣
 * next  0 򥻥åȤѤꤹ롣0 ʳξ
 * Ѥꤹ롣
 * @param ca оݤ COMACCESS
 * @param next 0 ξ硢ѡ0 ʳξϸѡ
 */
void caStart(COMACCESS *ca) {
  ca->fd = open(ca->DEVICENAME, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (ca->fd < 0) {perror(ca->DEVICENAME); exit(-1); }
   
  /* ߤΥꥢݡȤ򤵤*/
  tcgetattr(ca->fd, &(ca->oldtio)); 
  memset(&(ca->newtio), 0, sizeof(ca->newtio));
   
/* 
  BAUDRATE: ܡ졼Ȥꡥcfsetispeed  cfsetospeed ѤǤ롥
  CRTSCTS : ϤΥϡɥե (ɬפʷƤƤ륱
            ֥ȤΤߡSerial-HOWTO 7Ϥ򻲾ȤΤ)
  CS8     : 8n1 (8 ӥåȡΥѥƥȥåץӥå 1)
  CLOCAL  : ³ǥʤ
  CREAD   : ʸ(receiving characters)ͭˤ롥
*/
  ca->newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;

  /* IGNPAR  : ignore bytes with parity errors */
  /* (ѥƥ顼Υǡ̵뤹 )*/
  ca->newtio.c_iflag = IGNPAR;

  /* Raw output (Raw⡼ɤǤν)*/
  ca->newtio.c_oflag = 0;
  
  /* Set input mode (non-canonical,no echo,....) */
  ca->newtio.c_lflag = 0;
  
  ca->newtio.c_cc[VINTR]    = 0x03;     /* Ctrl-c */
  ca->newtio.c_cc[VQUIT]    = 0x1C;     /* Ctrl-\ */
  ca->newtio.c_cc[VERASE]   = 0x7F;     /* del */
  ca->newtio.c_cc[VKILL]    = 0x15;     /* @ */
  ca->newtio.c_cc[VEOF]     = 0x04;     /* Ctrl-d */
  ca->newtio.c_cc[VTIME]    = 0x00;     /* 饯֥ޤȤʤ */
  ca->newtio.c_cc[VMIN]     = 0x01;     /* 1ʸޤǡɤ߹ߤ֥å */
  ca->newtio.c_cc[VSWTC]    = 0x00;     /* '\0' */
  ca->newtio.c_cc[VSTART]   = 0x11;     /* Ctrl-q */ 
  ca->newtio.c_cc[VSTOP]    = 0x13;     /* Ctrl-s */
  ca->newtio.c_cc[VSUSP]    = 0x1A;     /* Ctrl-z */
  ca->newtio.c_cc[VEOL]     = 0x00;     /* '\0' */
  ca->newtio.c_cc[VREPRINT] = 0x12;     /* Ctrl-r */
  ca->newtio.c_cc[VDISCARD] = 0x0F;     /* Ctrl-u */
  ca->newtio.c_cc[VWERASE]  = 0x17;     /* Ctrl-w */
  ca->newtio.c_cc[VLNEXT]   = 0x16;     /* Ctrl-v */
  ca->newtio.c_cc[VEOL2]    = 0x00;     /* '\0' */

  tcflush(ca->fd, TCIFLUSH);
  tcsetattr(ca->fd,TCSANOW,&(ca->newtio));
}

/**
 * ca Ĥ롣
 * @param оݤCOMACCESS
 */
void caClose(COMACCESS *ca) {
  /* ݡȤץ೫ϻΤΤ᤹ */
  tcsetattr(ca->fd,TCSANOW,&(ca->oldtio));
   
  close(ca->fd);
}

/**
 * Ԥդ read() 
 * processing_pending() ¹Ԥ
 * ڥǥ󥰤Ƥ٥Ȥ롣
 * Ǥä -2 ֤
 * @param fd ե롦ǥץ
 * @param buf ʸǼХåե
 * @param size ϤԤХȿ
 * @return ξ read() ֤ͤΤޤޤ֤
 *         ԤǥǤä -2 ֤
 */
ssize_t wait_read(int fd, void *buf, size_t size) {
  int ret;

  while ( 1 ) {
    ret = read(fd, buf, size);
    if ( 0 < ret ) {
      return ret;
    } else {
      /* Ϥʤä */
      extern GAME g_game;
      if ( g_game.game_status == 1 ) {
	/* Ǥä롼פȴ롣 */
	return -2;
      }
      /* GUI νԤˤʤäƤ륤٥Ȥ */
      processing_pending();
    }
  }
}

/**
 * 롣ʸ buf ˳Ǽ롣buf  BUFSIZ ʾ礭
 * ꤬ݤƤ뤳ȤˤƤ롣
 * ʸ LF(16ʤ0A) ޤɤ³롣
 * @param ca оݤCOMACCESS
 * @param buf ʸǼ
 * @return 0 ξ1 ξäƼʤäƤ
 */
int caAccept(COMACCESS *ca, char *buf) {
  int i;
   
  for ( i=0; i<BUFSIZ; i++ ) {
    if ( wait_read(ca->fd, buf+i, 1) == -2 ) {
      /* ǤääƤ */
      return 1;
    }
    if ( buf[i] == 0x0D ) {
      i--;
      continue;
    }
    if ( buf[i] == 0x0A )
      break;
  }
  buf[i] = '\0';
  
  return 0;
}

/**
 * ʸ롣
 * @param ca оݤCOMACCESS
 * @param shr ʸ
 */
int caSend(COMACCESS *ca, char *str) {
  write(ca->fd, str, strlen(str));
  
  return 0;
}

#endif /* USE_SERIAL */

==> daemonshogi-0.1.4.tar.gz: daemonshogi-0.1.4/src/si/comaccess.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * comaccess.c
 * $Source: /cvsroot/daemonshogi/daemonshogi/src/si/comaccess.c,v $
 * $Id: comaccess.c,v 1.2 2005/12/12 02:52:11 tokita Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include "comaccess.h"
#include "si.h"
#include "ui.h"

#ifdef USE_SERIAL

#ifndef VSWTC
#define VSWTC 7
#endif /* VSWTC */

ssize_t wait_read(int fd, void *buf, size_t size);

/**
 * ݤCOMACCESSΥݥ󥿤֤
 * @return COMACCESSEΥݥ
 */
COMACCESS *newCOMACCESS(void) {
  COMACCESS *ca;

  ca = (COMACCESS *) malloc(sizeof(COMACCESS));
  if (ca == NULL) {
    printf("No enough memory. In tree.c/newTREE()");
    abort();
  }

  initCOMACCESS(ca);

  return ca;
}

/**
 * ca Υ롣
 * @param bo оݤCOMACCESS
 */
void freeCOMACCESS(COMACCESS *ca) {
#ifdef DEBUG
  assert( ca != NULL );
#endif /* DEBUG */
   
  free(ca); 
}

/**
 * ca 롣
 * @param ca оݤCOMACCESS
 */
void initCOMACCESS(COMACCESS *ca) {
   static const char s[] = "/dev/cua0";
   
   strncpy(ca->DEVICENAME, s, strlen(s));
   ca->DEVICENAME[strlen(s)] = '\0';
}

/**
 * ǥХ̾򥻥åȤ롣
 * @param ca оݤCOMACCESS
 * @param str ǥХ̾
 */
void caSetDEVICENAME(COMACCESS *ca, char *str) {
   strncpy(ca->DEVICENAME, str, strlen(str));
   ca->DEVICENAME[strlen(str)] = '\0';
}

/**
 * ̿жɤϤ褦ꤹ롣
 * next  0 򥻥åȤѤꤹ롣0 ʳξ
 * Ѥꤹ롣
 * @param ca оݤ COMACCESS
 * @param next 0 ξ硢ѡ0 ʳξϸѡ
 */
void caStart(COMACCESS *ca) {
  ca->fd = open(ca->DEVICENAME, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (ca->fd < 0) {perror(ca->DEVICENAME); exit(-1); }
   
  /* ߤΥꥢݡȤ򤵤*/
  tcgetattr(ca->fd, &(ca->oldtio)); 
  memset(&(ca->newtio), 0, sizeof(ca->newtio));
   
/* 
  BAUDRATE: ܡ졼Ȥꡥcfsetispeed  cfsetospeed ѤǤ롥
  CRTSCTS : ϤΥϡɥե (ɬפʷƤƤ륱
            ֥ȤΤߡSerial-HOWTO 7Ϥ򻲾ȤΤ)
  CS8     : 8n1 (8 ӥåȡΥѥƥȥåץӥå 1)
  CLOCAL  : ³ǥʤ
  CREAD   : ʸ(receiving characters)ͭˤ롥
*/
  ca->newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;

  /* IGNPAR  : ignore bytes with parity errors */
  /* (ѥƥ顼Υǡ̵뤹 )*/
  ca->newtio.c_iflag = IGNPAR;

  /* Raw output (Raw⡼ɤǤν)*/
  ca->newtio.c_oflag = 0;
  
  /* Set input mode (non-canonical,no echo,....) */
  ca->newtio.c_lflag = 0;
  
  ca->newtio.c_cc[VINTR]    = 0x03;     /* Ctrl-c */
  ca->newtio.c_cc[VQUIT]    = 0x1C;     /* Ctrl-\ */
  ca->newtio.c_cc[VERASE]   = 0x7F;     /* del */
  ca->newtio.c_cc[VKILL]    = 0x15;     /* @ */
  ca->newtio.c_cc[VEOF]     = 0x04;     /* Ctrl-d */
  ca->newtio.c_cc[VTIME]    = 0x00;     /* 饯֥ޤȤʤ */
  ca->newtio.c_cc[VMIN]     = 0x01;     /* 1ʸޤǡɤ߹ߤ֥å */
  ca->newtio.c_cc[VSWTC]    = 0x00;     /* '\0' */
  ca->newtio.c_cc[VSTART]   = 0x11;     /* Ctrl-q */ 
  ca->newtio.c_cc[VSTOP]    = 0x13;     /* Ctrl-s */
  ca->newtio.c_cc[VSUSP]    = 0x1A;     /* Ctrl-z */
  ca->newtio.c_cc[VEOL]     = 0x00;     /* '\0' */
  ca->newtio.c_cc[VREPRINT] = 0x12;     /* Ctrl-r */
  ca->newtio.c_cc[VDISCARD] = 0x0F;     /* Ctrl-u */
  ca->newtio.c_cc[VWERASE]  = 0x17;     /* Ctrl-w */
  ca->newtio.c_cc[VLNEXT]   = 0x16;     /* Ctrl-v */
  ca->newtio.c_cc[VEOL2]    = 0x00;     /* '\0' */

  tcflush(ca->fd, TCIFLUSH);
  tcsetattr(ca->fd,TCSANOW,&(ca->newtio));
}

/**
 * ca Ĥ롣
 * @param оݤCOMACCESS
 */
void caClose(COMACCESS *ca) {
  /* ݡȤץ೫ϻΤΤ᤹ */
  tcsetattr(ca->fd,TCSANOW,&(ca->oldtio));
   
  close(ca->fd);
}

/**
 * Ԥդ read() 
 * processing_pending() ¹Ԥ
 * ڥǥ󥰤Ƥ٥Ȥ롣
 * Ǥä -2 ֤
 * @param fd ե롦ǥץ
 * @param buf ʸǼХåե
 * @param size ϤԤХȿ
 * @return ξ read() ֤ͤΤޤޤ֤
 *         ԤǥǤä -2 ֤
 */
ssize_t wait_read(int fd, void *buf, size_t size) {
  int ret;

  while ( 1 ) {
    ret = read(fd, buf, size);
    if ( 0 < ret ) {
      return ret;
    } else {
      /* Ϥʤä */
      extern GAME g_game;
      if ( g_game.game_status == 1 ) {
	/* Ǥä롼פȴ롣 */
	return -2;
      }
      /* GUI νԤˤʤäƤ륤٥Ȥ */
      processing_pending();
    }
  }
}

/**
 * 롣ʸ buf ˳Ǽ롣buf  BUFSIZ ʾ礭
 * ꤬ݤƤ뤳ȤˤƤ롣
 * ʸ LF(16ʤ0A) ޤɤ³롣
 * @param ca оݤCOMACCESS
 * @param buf ʸǼ
 * @return 0 ξ1 ξäƼʤäƤ
 */
int caAccept(COMACCESS *ca, char *buf) {
  int i;
   
  for ( i=0; i<BUFSIZ; i++ ) {
    if ( wait_read(ca->fd, buf+i, 1) == -2 ) {
      /* ǤääƤ */
      return 1;
    }
    if ( buf[i] == 0x0D ) {
      i--;
      continue;
    }
    if ( buf[i] == 0x0A )
      break;
  }
  buf[i] = '\0';
  
  return 0;
}

/**
 * ʸ롣
 * @param ca оݤCOMACCESS
 * @param shr ʸ
 */
int caSend(COMACCESS *ca, char *str) {
  return write(ca->fd, str, strlen(str));;
}

#endif /* USE_SERIAL */
